package org.bitbucket.bradleysmithllc.etlagent.informatica.handler;

import com.fasterxml.jackson.databind.JsonNode;
import org.apache.commons.io.FileUtils;
import org.bitbucket.bradleysmithllc.etlagent.dto.GenericResponseDTO;
import org.bitbucket.bradleysmithllc.etlagent.informatica.dto.json.informatica.import_workflow_from_xml.ImportWorkflowFromXmlRequest;
import org.bitbucket.bradleysmithllc.etlagent.informatica.provider.InformaticaDomain;
import org.bitbucket.bradleysmithllc.etlagent.informatica.provider.InformaticaIntegrationService;
import org.bitbucket.bradleysmithllc.etlagent.informatica.provider.InformaticaRepository;
import org.bitbucket.bradleysmithllc.etlagent.informatica.provider.InformaticaRepositoryClient;
import org.bitbucket.bradleysmithllc.etlunit.RuntimeSupport;

import java.io.File;
import java.util.List;

public class ImportWorkflowFromXml extends AbstractHandler<ImportWorkflowFromXmlRequest, GenericResponseDTO>
{
	@Override
	public String getId() {
		return "importWorkflowFromXml";
	}

	@Override
	public ImportWorkflowFromXmlRequest getRequestContainerObject() {
		return new ImportWorkflowFromXmlRequest();
	}

	@Override
	public GenericResponseDTO process(final JsonNode request, final ImportWorkflowFromXmlRequest container) throws Exception {
		System.out.println("Importing workflow");

		String domain = container.getInformaticaDomain();

		String repo = container.getInformaticaRepository();

		// resolve to an informatica domain

		InformaticaDomain informaticaDomain = informaticaConfiguration.getDefaultDomain();

		if (domain != null)
		{
			System.out.println("No domain specified - using the default domain");

			informaticaDomain = informaticaConfiguration.getDomain(domain);

			if (informaticaDomain == null)
			{
				throw new IllegalArgumentException("Domain '" + domain + "' not found");
			}
		}
		else
		{
			System.out.println("No domain specified - using the default domain " + informaticaDomain.getDomainName());
		}

		InformaticaRepository informaticaRepository = null;

		if (repo == null)
		{
			informaticaRepository = informaticaDomain.getDefaultRepository();
		}
		else
		{
			informaticaRepository = informaticaDomain.getRepository(repo);
		}

		if (informaticaRepository == null)
		{
			throw new IllegalArgumentException("Domain '" + domain + "' does not contain repository '" + repo + "'");
		}

		useRepository(informaticaRepository, new RepositoryVisitor() {
			@Override
			public void useClient(InformaticaRepositoryClient client, RuntimeSupport runtimeSupport) throws Exception {
				InformaticaDomain informaticaDomain1 = client.getInformaticaDomain();
				InformaticaIntegrationService iintsvc = informaticaDomain1.getDefaultIntegrationService();

				if (container.getInformaticaIntegrationService() != null)
				{
					iintsvc = informaticaDomain1.getIntegrationService(container.getInformaticaIntegrationService());
				}

				if (iintsvc == null)
				{
					throw new IllegalArgumentException("Domain '" + informaticaDomain1.getDomainName() + "' does not contain integration service '" + container.getInformaticaIntegrationService() + "'");
				}

				String fileName = container.getWorkflowName();

				JsonNode requestUID = request.get("request-uid");

				if (requestUID != null && !requestUID.isNull())
				{
					fileName = requestUID.asText() + "_" + fileName;
				}

				File temp = runtimeSupport.createTempFile(fileName);

				FileUtils.writeStringToFile(temp, container.getWorkflowXml());

				List<String> fl = container.getFolderList();

				if (fl != null && fl.size() != 0)
				{
					client.importWorkflowFromXml(
						container.getFolderPrefix(),
						iintsvc,
						temp,
						fl
					);
				}
				else
				{
					client.importWorkflowFromXml(
						container.getFolderPrefix(),
						iintsvc,
						temp
					);
				}

				temp.delete();
			}
		});

		return null;
		//return JsonLoader.fromString("{\"response\": \"okay\"}");
	}
}