package org.bitbucket.bradleysmithllc.etlagent.informatica.provider;

import org.bitbucket.bradleysmithllc.etlagent.informatica.client.AgentClient;
import org.bitbucket.bradleysmithllc.etlagent.resources.ContextBase;
import org.bitbucket.bradleysmithllc.etlunit.RuntimeSupport;

public class InformaticaIntegrationService {
	private final String name;
	private final InformaticaDomain informaticaDomain;
	private final InformaticaRepository informaticaRepository;
	private final RuntimeSupport runtimeSupport;

	private int numRequestsSent = 0;

	private InformaticaIntegrationServiceClient informaticaIntegrationServiceClient;

	public InformaticaIntegrationService(String name, InformaticaRepository repository, RuntimeSupport runtimeSupport) {
		this.name = name;
		informaticaRepository = repository;
		informaticaDomain = informaticaRepository.getInformaticaDomain();
		this.runtimeSupport = runtimeSupport;

		informaticaIntegrationServiceClient = newInformaticaIntegrationServiceClient();
	}

	public String getIntegrationServiceName() {
		return name;
	}

	public InformaticaDomain getInformaticaDomain() {
		return informaticaDomain;
	}

	public InformaticaRepository getInformaticaRepository() {
		return informaticaRepository;
	}

	public InformaticaIntegrationServiceClient getInformaticaIntegrationServiceClient() {
		return informaticaIntegrationServiceClient;
	}

	public InformaticaIntegrationServiceClient newInformaticaIntegrationServiceClient() {
		switch (informaticaRepository.getInformaticaDomain().getClientType()) {
			case AGENT:
				return new AgentClient(this, runtimeSupport.getProjectUID());
			case LOCAL:
				return new InformaticaRepositoryClientImpl(informaticaRepository, this, informaticaDomain.getInformaticaBinDir(), runtimeSupport, runtimeSupport.getApplicationLog());
			case MOCK:
				return ClientProxyFactory.getInformaticaIntegrationServiceClientProxy(this, runtimeSupport);
		}

		throw new Error();
	}

	void request()
	{
		numRequestsSent++;
		informaticaDomain.request();
		informaticaRepository.request();
	}

	public int getNumRequestsSent() {
		return numRequestsSent;
	}

	@Override
	public String toString() {
		return "InformaticaIntegrationService{" +
				"name='" + name + '\'' +
				", informaticaDomain=" + informaticaDomain +
				", informaticaRepository=" + informaticaRepository +
				'}';
	}
}
